<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;
/**
 * 使用tinypng压缩上传图片，支持多组apikey随机切换
 * 
 * @package TinyPNG 
 * @author muchen
 * @version 1.0.0
 * @link http://typecho.org
 */
require_once("lib/Tinify/Exception.php");
require_once("lib/Tinify/ResultMeta.php");
require_once("lib/Tinify/Result.php");
require_once("lib/Tinify/Source.php");
require_once("lib/Tinify/Client.php");
require_once("lib/Tinify.php");
class TinyPNG_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     * 
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        Typecho_Plugin::factory('admin/footer.php')->end = array('TinyPNG_Plugin', 'render');
        Typecho_Plugin::factory('Widget_Upload')->uploadHandle = array('TinyPNG_Plugin', 'uploadHandle');
    }
    
    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     * 
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate(){}
    
    /**
     * 获取插件配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        /** 分类名称 */
        $name = new Typecho_Widget_Helper_Form_Element_Textarea('apikey', NULL, '', _t('apiKey'), _t('一行一个'));
        $form->addInput($name);
    }
    
    /**
     * 个人用户的配置面板
     * 
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form){
    }
    /**
     * 插件实现方法
     * 
     * @access public
     * @return void
     */
    public static function render()
    {
        $apikey = Typecho_Widget::widget('Widget_Options')->plugin("TinyPNG")->apikey;
        $html = '<div class="p" style="border: 1px dashed #D9D9D6;background-color: #FFF;color: #999;ont-size: .92857em;text-align: center;list-style: none;padding: 10px 0;">';
        if ($apikey) {
            $arr = explode(PHP_EOL, $apikey);
            foreach ($arr as $key => $value) {
                
                try {
                    $tinify = new \Tinify\Tinify;
                    $tinify -> setKey($value);
                    $tinify->getClient()->request("post", "/shrink", '');
                } catch (AccountException $err) {
                    if ($err->status != 429) throw $err;
                } catch (ClientException $err) {
                    return true;
                } catch(\Tinify\Exception $e) {
                   
                }
                $compressionsThisMonth[$key] = $tinify->getCompressionCount();
                $html .= '<p>';
                $html .= 'key' . $key . '已用' . $compressionsThisMonth[$key] . '次';
                $html .= '</p>';
            }
        }
        $html .= '</div>';
        echo '<script>' .
            '$(function(){' .
                '$("#upload-panel").before(\'' . $html . '\')'.
            '})'.
            '</script>';
    }
    /**
     * 获取安全的文件名 
     * 
     * @param string $name 
     * @static
     * @access private
     * @return string
     */
    private static function getSafeName(&$name)
    {
        $name = str_replace(array('"', '<', '>'), '', $name);
        $name = str_replace('\\', '/', $name);
        $name = false === strpos($name, '/') ? ('a' . $name) : str_replace('/', '/a', $name);
        $info = pathinfo($name);
        $name = substr($info['basename'], 1);
    
        return isset($info['extension']) ? strtolower($info['extension']) : '';
    }
    /**
     * 创建上传路径
     *
     * @access private
     * @param string $path 路径
     * @return boolean
     */
    private static function makeUploadDir($path)
    {
        $path = preg_replace("/\\\+/", '/', $path);
        $current = rtrim($path, '/');
        $last = $current;

        while (!is_dir($current) && false !== strpos($path, '/')) {
            $last = $current;
            $current = dirname($current);
        }

        if ($last == $current) {
            return true;
        }

        if (!@mkdir($last)) {
            return false;
        }

        $stat = @stat($last);
        $perms = $stat['mode'] & 0007777;
        @chmod($last, $perms);

        return self::makeUploadDir($path);
    }
    /**
     *
     * @access public
     * @param array $file 上传的文件
     * @return mixed
     */
    public static function uploadHandle($file)
    {
        if (empty($file['name'])) {
            return false;
        }

        $ext = self::getSafeName($file['name']);

        if (!Widget_Upload::checkFileType($ext) || Typecho_Common::isAppEngine()) {
            return false;
        }

        $date = new Typecho_Date();
        $path = Typecho_Common::url(defined('__TYPECHO_UPLOAD_DIR__') ? __TYPECHO_UPLOAD_DIR__ : Widget_Upload::UPLOAD_DIR,
            defined('__TYPECHO_UPLOAD_ROOT_DIR__') ? __TYPECHO_UPLOAD_ROOT_DIR__ : __TYPECHO_ROOT_DIR__) 
            . '/' . $date->year . '/' . $date->month;

        //创建上传目录
        if (!is_dir($path)) {
            if (!self::makeUploadDir($path)) {
                return false;
            }
        }

        //获取文件名
        $fileName = sprintf('%u', crc32(uniqid())) . '.' . $ext;
        $path = $path . '/' . $fileName;
        $isImage = in_array($ext, array("jpg", "png", "jpeg", "gif"));
        
        $apikey = Typecho_Widget::widget('Widget_Options')->plugin("TinyPNG")->apikey;
        if ($isImage && $apikey) {
            $arr = explode(PHP_EOL, $apikey);
            $key = array_rand($arr);
            $apikey = $arr[$key];
            
            \Tinify\setKey($apikey);
            
            if (isset($file['tmp_name'])) {
                //移动上传文件
                try {
                    $source = \Tinify\fromFile($file['tmp_name']);
                    $source->toFile($path);
                    $file['size'] = filesize($path);
                } catch(\Tinify\Exception $e) {
                    unset($file['size']);
                    $isImage = false;
                }
                
            } else if (isset($file['bytes'])) {
                
                try {
                    $resultData = \Tinify\fromBuffer($file['bytes'])->toFile($path);
                    $file['size'] = filesize($path);
                } catch(\Tinify\Exception $e) {
                    unset($file['size']);
                    $isImage = false;
                }
                //直接写入文件
                // if (!file_put_contents($path, $file['bytes'])) {
                //     return false;
                // }
            } else {
                return false;
            }
        }
        
        if (!$isImage) {
            
            if (isset($file['tmp_name'])) {
    
                //移动上传文件
                if (!@move_uploaded_file($file['tmp_name'], $path)) {
                    return false;
                }
            } else if (isset($file['bytes'])) {
    
                //直接写入文件
                if (!file_put_contents($path, $file['bytes'])) {
                    return false;
                }
            } else {
                return false;
            }
        }

        if (!isset($file['size'])) {
            $file['size'] = filesize($path);
        }

        //返回相对存储路径
        return array(
            'name' => $file['name'],
            'path' => (defined('__TYPECHO_UPLOAD_DIR__') ? __TYPECHO_UPLOAD_DIR__ : Widget_Upload::UPLOAD_DIR) 
                . '/' . $date->year . '/' . $date->month . '/' . $fileName,
            'size' => $file['size'],
            'type' => $ext,
            'mime' => Typecho_Common::mimeContentType($path)
        );
    }
}
